part of masamune;

extension WidgetRefValueNotifierExtensions on WidgetRef {
  ValueNotifier<T> useValueNotifier<T>(String key, T defaultValue) {
    final container = get<_ValueNotifierContainer<T>?>(key, null);
    if (container == null || container.defaultValue != defaultValue) {
      container?.dispose();
      final updated = this[key] = _ValueNotifierContainer<T>(
        value: ValueNotifier<T>(defaultValue),
        ref: this,
        defaultValue: defaultValue,
      );
      return updated.value;
    }
    return container.value;
  }
}

@immutable
class _ValueNotifierContainer<T> extends DisposableContainer {
  _ValueNotifierContainer({
    required this.value,
    required this.ref,
    required this.defaultValue,
  }) {
    value.addListener(_handledOnUpdate);
  }

  final ValueNotifier<T> value;
  final WidgetRef ref;
  final T defaultValue;

  @override
  void dispose() {
    value.removeListener(_handledOnUpdate);
    super.dispose();
  }

  void _handledOnUpdate() {
    ref.rebuild();
  }
}
